<?php

namespace App\Service;

class IFoodMenu
{
    private $iFood;

    public function __construct(\App\Integration\IFood $iFood)
    {
        $this->iFood = $iFood;
    }

    function formatStatus($originalStatus)
    {
        return $originalStatus === "AVAILABLE" ? "Disponível" : "Indisponível";
    }

    function formatDate($originalDate)
    {
        $originalDate = date_create($originalDate);
        return date_format($originalDate, "d-m-Y H:i:s");
    }

    public function getMenuId($merchantId, $accessToken = null)
    {
        $result = $this->iFood->getMenuId($merchantId, $accessToken);

        foreach ($result as &$item) {
            $item["status"] = $this->formatStatus($item["status"]);
            $item["modifiedAt"] = $this->formatDate($item["modifiedAt"]);
        }

        return $result;
    }

    public function GetCategory($menu)
    {
        $result = [];
        foreach ($menu as $category) {
            $categoryInfo = [
                "name"     => $category["name"],
                "id"       => $category["id"],
                "status"   => $this->formatStatus($category["status"]),
                "sequence" => $category["sequence"],
                "index"    => $category["index"],
                "template" => $category["template"],
            ];

            $result[] = $categoryInfo;
        }
        return $result;
    }

    private function requestIFoodItems($params) {
        $search = isset($params["search"]) ? $params["search"] : "";
        $menuId = isset($params["menuID"]) ? strval($params["menuID"]) : "";
        $merchantId = isset($params["merchantId"]) ? strval($params["merchantId"]) : "";

        $token = $this->iFood->getAuthToken();
        $menu = $this->iFood->getMenu($merchantId, $menuId, $token);

        return $menu;
    }

    public function items($params) {
        $getIFoodItems = $this->requestIFoodItems($params);
        $result = array();
        if (!empty($getIFoodItems)) {
            foreach ($getIFoodItems as $category) {
                $items = array();
                if (isset($category["items"])) {
                    foreach ($category["items"] as &$item) {
                        $this->verifyItemType($item);
                        $this->processOptions($item);
                        $items[] = $item;
                    }
                }
                $items = $this->buildTree($items);
                $result[] = [
                    "name"     => $category["name"],
                    "id"       => $category["id"],
                    "status"   => $this->formatStatus($category["status"]),
                    "sequence" => $category["sequence"],
                    "index"    => $category["index"],
                    "template" => $category["template"],
                    "items"    => $items,
                ];
            }
            return $result;
        }
    }

    private function verifyItemType(&$item) {
        $externalCode = $item['externalCode'] ?? '';
        $price = $item["price"]["value"] ?? 0;

        if(!$item['hasOptionGroups']) {
            $item['itemType'] = 'simple';
        } else {
            if (empty($externalCode) && empty($price)) {
                $item['itemType'] = 'combo';
            } elseif (!empty($externalCode) && !strstr($externalCode, "|") && $price >= 0.01) {
                $item['itemType'] = 'withAddons';
            } else {
                $item['itemType'] = 'invalid';
            }
        }
    }

    private function processOptions(&$item) {
        if(isset($item['itemType']) && $item['itemType'] === 'simple') {
            $this->checkIsValidItemSimple($item);
            return;
        }

        foreach ($item['optionGroups'] as &$optionGroup) {
            $optionGroup['parent_id'] = $item['id'];
            $optionGroup['item_id'] = $item['id'];
            foreach ($optionGroup['options'] as &$option) {
                $option['parent_id'] = $optionGroup['id'];
                $option['item_id'] = $item['id'];
                $this->checkIsValidOption($item, $option);
            }
        }
    }

    private function checkIsValidItemSimple(&$item) {
        $price = $item['price']['value'] ?? 0;
        $externalCode = $item['externalCode'] ?? '';
        $item['isValid'] = ($price >= 0.01 && !empty($externalCode) && !strstr($externalCode, "|"));
        $item['hasInvalidOptions'] = !$item['isValid'];
    }

    private function checkIsValidOption(&$item, &$option) {
        $externalCode = $option['externalCode'] ?? '';
        $price = $option['price']['value'] ?? 0;
        $itemType = $item['itemType'];

        switch($itemType) {
            case 'combo':
                $option['isValid'] = $price >= 0.01 && !empty($externalCode) && !strstr($externalCode, "|");
                break;
            case 'withAddons':
                $option['isValid'] = (empty($price) && (strlen($option['externalCode']) === 5 && substr($option['externalCode'], 2, 1) === '|')) ||
                                     (!empty($price) && !empty($externalCode));
                break;
            default:
                $option['isValid'] = false;
                break;
        }

        if(!$option['isValid']) {
            $item['hasInvalidOptions'] = true;
        }
    }

    private function buildTree($items) {
        $result = [];
        if(!empty($items)) {
            foreach ($items as $item) {
                $isValid = isset($item['hasInvalidOptions']) ? !$item['hasInvalidOptions'] : true;
                $result[] = [
                                "id"                => $item['id'],
                                "name"              => $item['name'],
                                "sku"               => $item['externalCode'] ?? null,
                                "price"             => $item['price']['value'],
                                "itemType"          => $item['itemType'],
                                "isValid"           => $isValid,
                                "type"              => "I", // Item
                            ];
                if(isset($item['optionGroups'])) {
                    foreach($item['optionGroups'] as $group) {
                        $result[] = [
                                        "id"        => $group["id"],
                                        "parent_id" => $group['parent_id'],
                                        "item_id"   => $group['item_id'],
                                        "name"      => $group["name"],
                                        "isValid"   => $isValid,
                                    ];

                        if(isset($group['options'])) {
                            foreach($group['options'] as $option){
                                $result[] = [
                                    "id"           => $option['id'],
                                    "price"        => $option['price']['value'],
                                    "parent_id"    => $option['parent_id'],
                                    "item_id"      => $option['item_id'],
                                    "name"         => $option['name'],
                                    "sku"          => $option['externalCode'] ?? null,
                                    "type"         => "C", // Complemento
                                    "isValid"      => $isValid,
                                ];
                            }
                        }
                    }
                }
            }
        }
        return $result;
    }

    public function setOptionExternalCode($merchantId, $item, $token = null) {
        try {
            $data = [
                "optionId"                    => $item['id'],
                "externalCode"                => $item['finalSku'],
                "parentCustomizationOptionId" => null,
                "externalCodeByCatalog" => [
                    [
                      "externalCode"   => $item['finalSku'],
                      "catalogContext" => $item['menuId']
                    ]
                  ]
            ];
            $postData = http_build_query($data);

            $this->iFood->setOptionExternalCode($merchantId, $postData, $token);

            return [];
        } catch (\Exception $err) {
            throw new \Exception("Ocorreu um erro ao tentar atualizar esta opção.");
        }
    }
    /*
     * @TODO: Finalizar implementação para atualizar preço do item
     *

    public function updateItemPrice($accessToken, $merchantId, $item) {
        $data = [
            "itemId" => $item['menuId'],
            "price"  => [
              "value" => $item['finalPrice'],
              "scalePrices" => [
                [
                  "minQuantity" => 1,
                  "price" => $item['finalPrice'],
                ]
              ]
            ],
            "priceByCatalog" => [
              [
                "value" =>$item['finalPrice'],
                "catalogContext" => $item['menuId']
              ]
            ]
        ];

        $postData = http_build_query($data);

        $data = json_encode($data);

        $url = "https://merchant-api.ifood.com.br/catalog/v2.0/merchants/" . $merchantId . "/items/price";

        $curl = curl_init();

        curl_setopt_array($curl, [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "PATCH",
            CURLOPT_POSTFIELDS => $postData,
            CURLOPT_HTTPHEADER => ["Authorization: Bearer " . $accessToken],
        ]);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

        $curlResponse = curl_exec($curl);

        if (curl_errno($curl)) {
            throw new \Exception("Ocorreu um erro ao tentar atualizar o preço deste item.");
        }

        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        if ($httpCode != 200) {
            throw new \Exception($curlResponse);
        }

        curl_close($curl);

        return [];
    }
    */

    /*
     * @TODO: Finalizar implementação para atualizar preço de complementos
     *

    public function updateOptionPrice($accessToken, $merchantId, $item) {
        $data = [
            "itemId" => $item['id'],
            "price"  => [
                "value" => $item['finalPrice'],
                "scalePrices" => [
                    [
                        "price" => $item['finalPrice'],
                        "minQuantity" => 1
                    ]
                ],
            ],
            "priceByCatalog" => [
                [
                  "value"          => $item['finalPrice'],
                  "catalogContext" => $item['menuId']
                ]
            ]
        ];
        $postData = http_build_query($data);

        $data = json_encode($data);

        $url = "https://merchant-api.ifood.com.br/catalog/v2.0/merchants/" . $merchantId . "/options/price";

        $curl = curl_init();

        curl_setopt_array($curl, [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "PATCH",
            CURLOPT_POSTFIELDS => $postData,
            CURLOPT_HTTPHEADER => ["Authorization: Bearer " . $accessToken],
        ]);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

        $curlResponse = curl_exec($curl);

        if (curl_errno($curl)) {
            throw new \Exception("Ocorreu um erro ao tentar atualizar o preço deste item.");
        }

        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        if ($httpCode != 200) {
            throw new \Exception("Ocorreu um erro ao tentar atualizar o preço deste item.");
        }

        curl_close($curl);

        return [];
    }
    */

    public function setItemExternalCode($merchantId, $item, $token = null) {
        $data = [
            "itemId"                => $item['id'],
            "externalCode"          => $item['finalSku'],
            "externalCodeByCatalog" => [
                [
                  "externalCode"   => $item['finalSku'],
                  "catalogContext" => $item['menuId']
                ]
              ]
        ];

        $postData = http_build_query($data);

        $this->iFood->setItemExternalCode($merchantId, $postData, $token);
        return [];
    }
}